// naumann@stce.rwth-aachen.de

#include <iostream> 
#include <fstream> 
#include <cassert> 
#include <cmath> 
#include <vector> 
#include <limits>

template<typename T>
class model {
  const T eps=std::numeric_limits<T>::epsilon();
  std::vector<std::pair<T,T>> observations;
  T p;
  std::string filename;
public:
  T evaluate(T x) const { return p*x; }
  model(char* fn) : filename(fn) {
    std::ifstream ifs(filename); 
    ifs >> p; size_t m=0; ifs >> m; 
    T x,y; 
    for (size_t i=0;i<m;i++) {
      ifs >> x >> y; observations.push_back(std::make_pair(x,y));
    }
    ifs.close(); 
  }
  ~model() {
    std::ofstream ofs(filename); 
    ofs << p << std::endl << observations.size() << std::endl; 
    for (const auto& o : observations) 
      ofs << o.first << " " << o.second << std::endl;
    ofs.close(); 
  }
  void add_observation(const T& x, const T &y) {
    observations.push_back(std::make_pair(x,y));
  }
private:
  T error() const {
    T e=0;
    for (const auto& o : observations) 
      e+=pow(evaluate(o.first)-o.second,2);
    return e;
  }
  T d_error() {
    const T delta_p=(p!=0) ? sqrt(sqrt(eps))*fabs(p) : sqrt(sqrt(eps));
    p+=delta_p; T e=error(); p-=delta_p;
    return (e-error())/delta_p;
  }
  T dd_error() {
    const T delta_p=(p!=0) ? sqrt(sqrt(eps))*fabs(p) : sqrt(sqrt(eps));
    p+=delta_p; T e=d_error(); p-=delta_p;
    return (e-d_error())/delta_p;
  }
public:
  T calibrate() {
    while (fabs(d_error())>sqrt(eps)) 
      p-=d_error()/dd_error();
    return p;
  }
  void plot() const {
    std::ofstream dfs("data.plot"); 
    T max_x=0;
    for (const auto& o : observations) {
      dfs << o.first << " " << o.second << std::endl;
      max_x=std::max(o.first,max_x);
    }
    std::ofstream mfs("model.plot"); 
    for (T x=0;x<=max_x;x+=1e-1)
      mfs << x << " " << evaluate(x) << std::endl;
  }
};

int main(int argc, char *argv[]) {
  assert(argc==2); 
  model<double> p(argv[1]); 
  std::cout << "new observation:" << std::endl;
  double x,y;
  std::cout << "x="; std::cin >> x;
  std::cout << "y="; std::cin >> y;
  p.add_observation(x,y);
  std::cout << "new p=" << p.calibrate() << std::endl;
  p.plot();
  return 0;
}

